within IDEAS.Fluid.HeatExchangers.GroundHeatExchangers.Borefield.BaseClasses.BoreHoles.BaseClasses;
function singleUTubeResistances
  "Thermal resistances for single U-tube, according to Bauer et al. (2011)"
  // FIXME: if python is used, the results are slightly different. I guess the Dymola resultion is quite low.
  extends
    IDEAS.Fluid.HeatExchangers.GroundHeatExchangers.Borefield.BaseClasses.BoreHoles.BaseClasses.partialBoreholeResistances;

  // Outputs
  output Modelica.SIunits.ThermalResistance Rgb
    "Thermal resistance between grout zone and borehole wall";
  output Modelica.SIunits.ThermalResistance Rgg
    "Thermal resistance between the two grout zones";
  output Modelica.SIunits.ThermalResistance RCondGro
    "Thermal resistance between: pipe wall to capacity in grout";
protected
  Modelica.SIunits.ThermalResistance Rg
    "Thermal resistance between outer borehole wall and one tube";
  Modelica.SIunits.ThermalResistance Rar
    "Thermal resistance between the two pipe outer walls";

  Real Ra(unit="(m.K)/W")
    "Grout-to-grout resistance (2D) as defined by Hellstroem. Interaction between the different grout part";

  // Help variables
  Real sigma "Help variable as defined by Hellstroem";
  Real beta "Help variable as defined by Hellstroem";
  Real R_1delta_LS(unit="(m.K)/W")
    "One leg of the triangle resistance network, corresponding to the line source solution";
  Real R_1delta_MP(unit="(m.K)/W")
    "One leg of the triangle resistance network, corresponding to the multipole solution";
  Real Ra_LS(unit="(m.K)/W")
    "Grout-to-grout resistance calculated with the line-source approximation";

algorithm
  // ********** Rb and Ra from multipole **********
  // Help variables
  sigma :=(kFil - kSoi)/(kFil + kSoi);
  beta :=2*Modelica.Constants.pi*kFil*(RCondPipe+RConv);

  R_1delta_LS :=1/(2*pi*kFil)*(log(rBor/rTub) + log(rBor/(2*sha)) +
    sigma*log(rBor^4/(rBor^4 - sha^4))) + RCondPipe + RConv;

  R_1delta_LS :=1/(2*Modelica.Constants.pi*kFil)*(log(rBor/rTub) + log(rBor/(2*
    sha)) + sigma*log(rBor^4/(rBor^4 - sha^4))) + RCondPipe + RConv;

  R_1delta_MP :=R_1delta_LS - 1/(2*pi*kFil)*(rTub^2/
    (4*sha^2)*(1 - sigma*4*sha^4/(rBor^4 - sha^4))^2)/((1 + beta)/(1 - beta) +
    rTub^2/(4*sha^2)*(1 + sigma*16*sha^4*rBor^4/(rBor^4 - sha^4)^2));

  Ra_LS      :=1/(pi*kFil)*(log(2*sha/rTub) + sigma*log((
    rBor^2 + sha^2)/(rBor^2 - sha^2)))  + 2*(RCondPipe + RConv);

  //Rb and Ra
  Rb_internal :=if use_Rb then Rb else R_1delta_MP/2;
  Ra :=Ra_LS - 1/(Modelica.Constants.pi*kFil)*(rTub^2/(4*sha^2)*(1 + sigma*
    4*rBor^4*sha^2/(rBor^4 - sha^4))/((1 + beta)/(1 - beta) - rTub^2/(4*sha^2) +
    sigma*2*rTub^2*rBor^2*(rBor^4 + sha^4)/(rBor^4 - sha^4)^2));

  //Conversion of Rb (resp. Ra) to Rg (resp. Rar) of Bauer:
  Rg  :=(2*Rb_internal-RCondPipe-RConv)/hSeg;
  Rar :=(Ra-2*(RCondPipe + RConv))/hSeg;

/* **************** Simplification of Bauer for single U-tube ************************
  //Thermal resistance between: Outer wall and one tube
     Rg := Modelica.Math.acosh((rBor^2 + (rTub + eTub)^2 - sha^2)/(2*rBor*(rTub +
       eTub)))/(2*Modelica.Constants.pi*hSeg*kFil)*(1.601 - 0.888*sha/rBor);

  //Thermal resistance between: The two pipe outer walls
  Rar := Modelica.Math.acosh((2*sha^2 - (rTub + eTub)^2)/(rTub + eTub)^2)/(2*
       Modelica.Constants.pi*hSeg*kFil);
*************************************************************************************** */

  // ********** Resistances and capacity location according to Bauer **********
  while test == false and i <= 10 loop
    // Capacity location (with correction factor in case that the test is negative)
    x := Modelica.Math.log(sqrt(rBor^2 + 2*rTub^2)/(2*rTub))/
      Modelica.Math.log(rBor/(sqrt(2)*rTub))*((15 - i + 1)/15);

    //Thermal resistance between the grout zone and bore hole wall
    Rgb := (1 - x)*Rg;

    //Thermal resistance between the two grout zones
    Rgg := 2*Rgb*(Rar - 2*x*Rg)/(2*Rgb - Rar + 2*x*Rg);

    // Thermodynamic test to check if negative R values make sense. If not, decrease x-value.
    // fixme: the implemented is only for single U-tube BHE's.
    test := ((1/Rgg + 1/2/Rgb)^(-1) > 0);
    i := i + 1;
  end while;
  assert(test,
  "Maximum number of iterations exceeded. Check the borehole geometry.
  The tubes may be too close to the borehole wall.
  Input to the function 
  IDEAS.Fluid.HeatExchangers.Boreholes.BaseClasses.singleUTubeResistances
  is
           hSeg = " + String(hSeg) + " m
           rBor = " + String(rBor) + " m
           rTub = " + String(rTub) + " m
           eTub = " + String(eTub) + " m
           kSoi = " + String(kSoi) + " W/m/K
           kFil = " + String(kFil) + " W/m/K
           kTub = " + String(kTub) + " W/m/K
           i = "  + String(i) + "
  Computed x    = " + String(x) + " K/W
           Rgb  = " + String(Rgb) + " K/W
           Rgg  = " + String(Rgg) + " K/W");

  //Conduction resistance in grout from pipe wall to capacity in grout
  RCondGro := x*Rg + RCondPipe/hSeg;

  if printDebug then
    Modelica.Utilities.Streams.print("
Rb = " + String(Rb_internal) + " m K / W
RCondPipe = "+ String(RCondPipe) + " m K / W
RConv = " +String(RConv) +"m K / W
hSeg = " + String(hSeg) + " m
Rg = "+String(Rg) + " K / W
Ra = " + String(Ra)  + " m K / W
x = " + String(x) + "
i = "  + String(i));
  end if;
                                                        annotation (Diagram(graphics), Documentation(info="<html>
<p>
This model computes the different thermal resistances present in a single-U-tube borehole 
using the method of Bauer et al. [1].
It also computes the fluid-to-ground thermal resistance <i>R<sub>b</sub></i> 
and the grout-to-grout thermal resistance <i>R<sub>a</sub></i> 
as defined by Hellstroem [2] using the multipole method.
</p>
<p>
The figure below shows the thermal network set up by Bauer et al.
</p>
<p align=\"center\">
<img alt=\"image\" src=\"E:/work\\modelica/DaPModels/Images/Documentation/Bauer_singleUTube_small.png\"/>
</p>
<p>
The different resistances are calculated with following equations:</p>
<p align=\"center\">
<img alt=\"image\" src=\"E:/work\\modelica/DaPModels/Images/Documentation/Bauer_resistanceValues.PNG\"/>
</p>
<p>
Notice that each resistance each resistance still needs to be divided by 
the height of the borehole segment <i>h<sub>Seg</sub></i>.
</p>
<p>
The fluid-to-ground thermal resistance <i>R<sub>b</sub></i> and the grout-to-grout resistance <i>R<sub>a</sub></i> 
are calculated with the multipole method (Hellstroem (1991)) shown below.
</p>
<p>
<!-- If this is an equation, it needs to be typed, not an image -->
<img alt=\"image\" src=\"E:/work\\modelica/DaPModels/Images/Documentation/Rb_multipole.png\"/>
</p>
<p>
<!-- If this is an equation, it needs to be typed, not an image -->
<img alt=\"image\" src=\"E:/work\\modelica/DaPModels/Images/Documentation/Ra_multipole.png\"/>
</p>
<p>
where 
<!-- fixme: use greek symbols such as &lambda; -->
<i>lambda<sub>b</sub></i> and <i>lambda</i>are the conductivity of the filling material 
and of the ground respectively, 
<i>r<sub>p</sub></i> and <i>r<sub>b</sub></i> 
are the pipe and the borehole radius, 
<i>D</i> is the shank spacing (center of borehole to center of pipe), 
<i>R<sub>p</sub></i> is resistance from the fluid to the outside wall of the pipe, 
<i>r<sub>c</sub></i> is the radius at which the ground temperature is radially uniform and 
<i>Epsilon</i> can be neglected as it is close to zero.
</p>
<h4>References</h4>
<p>G. Hellstr&ouml;m. 
<i>Ground heat storage: thermal analyses of duct storage systems (Theory)</i>. 
Dept. of Mathematical Physics, University of Lund, Sweden, 1991.
</p>
<p>D. Bauer, W. Heidemann, H. M&uuml;ller-Steinhagen, and H.-J. G. Diersch. 
<i>Thermal resistance and capacity models for borehole heat exchangers</i>. 
International Journal Of Energy Research, 35:312&ndash;320, 2010.</p>
</html>", revisions="<html>
<p>
<ul>
<li>
February 14, 2014 by Michael Wetter:<br/>
Added an assert statement to test for non-physical values.
</li>
<li>
February 12, 2014, by Damien Picard:<br/>
Remove the flow dependency of the resistances, as this function calculates the conduction resistances only.
</li>
<li>
January 24, 2014, by Michael Wetter:<br/>
Revised implementation.
</li>
<li>
January 23, 2014, by Damien Picard:<br/>
First implementation.
</li>
</ul></p>
</html>"));
end singleUTubeResistances;
